package com.kccoder.fracture;

import java.util.Iterator;

/**
 * This is the core class in the Fracture library.  Here you can set/get the 'Core Mode' and execute 
 * multi-core loops either synchronously or asynchronously.
 * <br/><br/>
 * This software is distributed WITHOUT ANY WARRANTY, without even the 
 * implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * 
 * @author Cameron Shelton (http://kccoder.com)
 */

public class Fracture {
	private static CoreMode coreMode = CoreMode.ALL_CORES;
	
	/**
	 * Get the currently configured CoreMode.
	 * @return The currently configured CoreMode
	 */
	public static CoreMode getCoreMode() {return coreMode;}
	
	/**
	 * Confiure the Fracture library CoreMode.
	 * @param coreMode - the desired CoreMode
	 */
	public static void setCoreMode(CoreMode coreMode) {Fracture.coreMode = coreMode;}
	
	/**
	 * Process each array element on N cores.
	 * @param <T> Element type
	 * @param elements Element array
	 * @param processor Processor to process each element
	 */
	public static <T> void forEach(T[] elements, IProcessor<T> processor) {
		new ThreadedForEach<T>(elements, processor).loop();
	}
	
	/**
	 * Process each element present in the iterable's iterator on N cores.
	 * @param <T> Element type
	 * @param iterable Elements to be processed
	 * @param processor Processor to process each element
	 */	
	public static <T> void forEach(Iterable<T> iterable, IProcessor<T> processor) {
		forEach(iterable.iterator(), processor);
	}

	/**
	 * Process each element present in the iterator on N cores.
	 * @param <T> Element type
	 * @param iterator Elements to be processed
	 * @param processor Processor to process each element
	 */	
	public static <T> void forEach(Iterator<T> iterator, IProcessor<T> processor) {
		new ThreadedForEach<T>(iterator, processor).loop();
	}

	/**
	 * Essentially equivalent to for(int i = start; i < end; i+=increment) executed on N cores.
	 * @param start Starting value
	 * @param end Ending value
	 * @param increment Increment value
	 * @param processor Processor to process each integer
	 */
	public static void forEach(int start, int end, int increment, IProcessor<Integer> processor) {
		if (increment > 0) {
			forEach(new SequentialIterator(start, end, increment), processor);
		} else {
			forEach(new DecliningSequentialIterator(start, end, increment), processor);
		}
	}

	/**
	 * Process each array element on N cores. This method executes this process in the bacground and then returns.
	 * @param <T> Element type
	 * @param elements Element array
	 * @param processor Processor to process each element
	 */
	public static <T> void forEachSync(final T[] elements, final IProcessor<T> processor) {
		runSync(new Runnable() {public void run() {forEachSync(elements, processor);}});
	}
	
	/**
	 * Process each element present in the iterable's iterator on N cores. 
	 * This method executes this process in the bacground and then returns.
	 * @param <T> Element type
	 * @param iterable Elements to be processed
	 * @param processor Processor to process each element
	 */	
	public static <T> void forEachSync(final Iterable<T> iterable, final IProcessor<T> processor) {
		runSync(new Runnable() {public void run() {forEachSync(iterable, processor);}});
	}

	/**
	 * Process each element present in the iterator on N cores. 
	 * This method executes this process in the bacground and then returns.
	 * @param <T> Element type
	 * @param iterator Elements to be processed
	 * @param processor Processor to process each element
	 */	
	public static <T> void forEachSync(final Iterator<T> iterator, final IProcessor<T> processor) {
		runSync(new Runnable() {public void run() {forEachSync(iterator, processor);}});
	}
	
	/**
	 * Essentially equivalent to for(int i = start; i < end; i+=increment) executed on N cores. 
	 * This method executes this process in the bacground and then returns.
	 * @param start Starting value
	 * @param end Ending value
	 * @param increment Increment value
	 * @param processor Processor to process each integer
	 */
	public static void forEachSync(final int start, final int end, 
	final int increment, final IProcessor<Integer> processor) {
		runSync(new Runnable() {public void run() {forEachSync(start, end, increment, processor);}});	
	}
	
	private static void runSync(Runnable runnable) {
		Thread t = new Thread(runnable);
		t.setDaemon(true);
		t.start();
	}
	
}